require 'mkmf'
dir_config('rubycdio')
require "rbconfig"
if Config::CONFIG["host_os"] == "cygwin"
  $libs = append_library($libs, "winmm")  
  $libs = append_library($libs, "m")  
end
local_libs = `pkg-config libcdio --libs`
if local_libs
  $LOCAL_LIBS = local_libs.chomp 
else 
  $libs = append_library($libs, "cdio")
end
destdir='/usr/local/lib'
DESTDIR='/usr/local/lib'
CONFIG["prefix"] = '/usr/local'
create_makefile('rubycdio')
